@@ -6,7 +6,6 @@
 #ifndef XGBOOST_COMMON_COMMON_H_
 #define XGBOOST_COMMON_COMMON_H_
 
-#include <algorithm>  // for max
 #include <array>      // for array
 #include <cmath>      // for ceil
 #include <cstddef>    // for size_t
@@ -67,8 +66,20 @@ inline std::vector<std::string> Split(const std::string& s, char delim) {
   return ret;
 }
 
+/**
+ * @brief Add escapes for a UTF-8 string.
+ */
 void EscapeU8(std::string const &string, std::string *p_buffer);
 
+/**
+ * @brief Add escapes for a UTF-8 string with newly created buffer as return.
+ */
+inline std::string EscapeU8(std::string const &str) {
+  std::string buffer;
+  EscapeU8(str, &buffer);
+  return buffer;
+}
+
 template <typename T>
 XGBOOST_DEVICE T Max(T a, T b) {
   return a < b ? b : a;
@@ -164,10 +175,16 @@ inline void AssertGPUSupport() {
 #endif  // XGBOOST_USE_CUDA
 }
 
-inline void AssertOneAPISupport() {
-#ifndef XGBOOST_USE_ONEAPI
-    LOG(FATAL) << "XGBoost version not compiled with OneAPI support.";
-#endif  // XGBOOST_USE_ONEAPI
+inline void AssertNCCLSupport() {
+#if !defined(XGBOOST_USE_NCCL)
+    LOG(FATAL) << "XGBoost version not compiled with NCCL support.";
+#endif  // !defined(XGBOOST_USE_NCCL)
+}
+
+inline void AssertSYCLSupport() {
+#ifndef XGBOOST_USE_SYCL
+    LOG(FATAL) << "XGBoost version not compiled with SYCL support.";
+#endif  // XGBOOST_USE_SYCL
 }
 
 void SetDevice(std::int32_t device);
@@ -181,7 +198,7 @@ inline void SetDevice(std::int32_t device) {
 #endif
 
 /**
- * Last index of a group in a CSR style of index pointer.
+ * @brief Last index of a group in a CSR style of index pointer.
  */
 template <typename Indexable>
 XGBOOST_DEVICE size_t LastOf(size_t group, Indexable const &indptr) {
